import threading
import socket
import select
import time
import util
from Forward import CliForw
from BobSock import ConSock

##used to forward a local port to a specific i2p dest/b32
class Tunnel(threading.Thread):
    def __init__(self,public):
        self.public = public
        self.localsock = socket.socket()
        self.localsock.bind(("127.0.0.1",0))
        self.port = self.localsock.getsockname()[1]
        self.forport = util.RandPort()
        self.forw = CliForw(self.forport)
        self.relays = []
        threading.Thread.__init__(self)
        self.start()
    def run(self):
        self.done = False
        self.localsock.listen(10)
        self.localsock.settimeout(1.0)
        try:
            while True:
                time.sleep(0.001)
                try:
                    con, addr = self.localsock.accept()
                    self.relays.append(Relay(con,self.public,self.forport))
                except socket.timeout:
                    if self.done==True:
                        raise
        except:
            self.on_exit()
    def on_exit(self):
        for rlay in self.relays:
            rlay.close()
            rlay.join()
    def close(self):
        self.done = True
        self.forw.close()
        self.localsock.close()
#makes the connection to dest/b32 and relays traffic between local sock and bobsock
class Relay(threading.Thread):
    def __init__(self,sock,public,port):
        self.sock = sock
        self.public = public
        self.port = port
        threading.Thread.__init__(self)
        self.start()
    def run(self):
        try:
            self.consock = ConSock(self.public,self.port)
            self.sock.setblocking(0)
            self.consock.setblocking(0)
            while True:
                readable, sendable, errable = select.select([self.sock,self.consock],[],
                                                            [self.sock,self.consock],30)
                if errable == []:
                    for s in readable:
                        data = s.recv(1024)
                        if data == "":
                            raise socket.error
                        if s == self.sock:
                            self.consock.send(data)
                        else:
                            self.sock.send(data)
                else:
                    raise Exception(repr(errable))
        except:
            self.sock.close()
            try:
                self.consock.close()
            except:
                pass
    def close(self):
        self.sock.close()
        try:
            self.consock.close()
        except:
            pass